
/**
 * Convert a file input element into an inline file upload control.
 * <h4>Use </h4>
 *
 * <h5>Markup:</h5>
 *
 * <pre>
 *   <input class="upfile " name="tempFilename" type="file">
 * </pre>
 *
 * <h5>JavaScript</h5>
 *
 * <pre>
 * // no options
 * new AJS.InlineAttach("input:file");
 *
 * // with options
 * new AJS.InlineAttach({
 *      element: "input:file",
 *      issueId: 10022
 * });
 * </pre>
 *
 * @class InlineAttach
 * @module Controls
 * @requires AJS
 *
 */
AJS.InlineAttach = Class.extend({

    /**
     * Gets default options
     *
     * @method _getDefaultOptions
     * @private
     *
     */
    _getDefaultOptions: function () {
        return {
            postUrl: contextPath + "/secure/AttachTemporaryFile.jspa"
        };
    },

    /**
     * Creates an inline file attach control
     *
     * @constructor
     * @param options
     *
     * <dl>
     *  <dt>{String, jQuery} element</dt>
     *      <dd>Element defining the file input HTML element</dd>
     *      <dd><strong>Default: <em>input[type=file]</em></strong><dd>
     *  <dt>{Number} issueId</dt>
     *      <dd>The issue id to attach temporary files to</dd>
     *      <dd><strong>Default: <em>undefined</em></strong></dd>
     *  <dt>{Number} projectId</dt>
     *      <dd>The project to attach temporary files to (when creating a new issue)</dd>
     *      <dd><strong>Default: <em>undefined</em></strong></dd>
     * </dl>
     *
     */
    init: function (options) {

        var that = this;

        if (typeof options === "string") {
            options = {element: options};
        }
        options = options || {};

        this.options = AJS.$.extend(this._getDefaultOptions(), options);
        this.$container = AJS.$(this.options.element);
        this.$form = this.$container.closest("form");
        this.$container.change(function() {
            that.attachFile(AJS.$(this));
        });
    },

    /**
     * Gets the short filename of the file attached by the user.  All browsers except for IE will just
     * return the filename.  IE returns the absolute path, so we strip everything before the last \
     * from the filename in IE.
     *
     * @method _getFilename
     * @private
     * @param {String} fileName - the filename to cut
     * @return {String} the filename correct accross all browsers
     */
    _getFilename: function(fileName) {
        if(AJS.$.browser.msie && fileName.indexOf("\\") >= 0) {
            //IE returns an absolute path for the selected file, we however only want to display the
            //filename.
            fileName = fileName.substring(fileName.lastIndexOf("\\") + 1);
        }
        return fileName;
    },

    /**
     * Gets the atlassian_token from the form (using the passed in element to determine the current form).
     * If there is no 'atl_token' input on the form or the element is null, it will return the global one. 
     *
     * @param $element an element inside a form.
     */
    _getAtlToken: function($element){
        if ($element){
            var $form = $element.closest("form");
            var $atlToken = AJS.$("input[name='atl_token']", $form);
            if ($atlToken.length > 0){
                return $atlToken.val();
            }
        }

        return atl_token();
    },

    /**
     * The guts of this control.  This method is called on change() of the file input control.  It
     * takes the value, and creates a loading span with throbber to display for the user.
     *
     * It then creates a hidden multipart form with the file input and submits it to the server.
     * On success a checked checkbox for the uploaded file is shown.  On error a an error container
     * with any server errors is rendered.
     *
     * @method shorten
     * @param {jQuery} $input - The input element jQuery object whose value was just changed
     *
     */
    attachFile: function($input) {
        var that = this,
                $fileInputContainer = $input.parent(),
                $originalFileInput = this._renders.fileInput($input),
                $attachForm = this._renders.attachForm(this.options.postUrl).append($input),
                $loadingSpan = this._renders.loadingSpan(this._getFilename($input.val())),
                postData = { atl_token: this._getAtlToken($fileInputContainer) };

        $originalFileInput.change(function() {
            that.attachFile(AJS.$(this));
        });
        //clear any errors when uploading the next file
        $fileInputContainer.parent().find("div.error").remove();
        AJS.$("body").append($attachForm);
        if(this.options.issueId) {
            postData.id = this.options.issueId;
        } else {
            postData.create = true;
            postData.projectId = this.options.projectId;
        }

        $attachForm.ajaxForm({
            dataType: "json",
            data: postData,
            //set the timeout to infinity since it could take a long time to upload large files (JRADEV-3455)
            timeout:0,
            cleanupAfterSubmit: function() {
                //first remove the loading span for this upload!
                $loadingSpan.remove();

                //JRADEV-3554: Only enable the submit button if there's nothing else loading any longer!
                if(that.$form.find(".loading.file").length === 0) {
                    that.$form.find("input[type=submit]").removeAttr("disabled");
                }
                $attachForm.remove();
            },
            beforeSubmit: function () {
                $fileInputContainer.append($loadingSpan);
                if ($fileInputContainer.is("td")){
                    $fileInputContainer.append(AJS.$("<div class='field-group'/>").append($originalFileInput));
                } else{
                    $fileInputContainer.after(AJS.$("<div class='field-group'/>").append($originalFileInput));                                        
                }

                if(AJS.$.browser.msie) {
                    //IE being the usual pain that it is wont focus unless there's this timeout.
                    setTimeout(function() {$originalFileInput.focus();}, 0);
                } else {
                    $originalFileInput.focus();
                }
                that.$form.find("input[type=submit]").attr("disabled", "true");
            },
            error: function (xhr) {
                if(xhr && xhr.responseText && xhr.responseText.indexOf("SecurityTokenMissing") >= 0) {
                    $loadingSpan.after(that._renders.errors(AJS.params.attachmentsXsrfTimeout));
                } else {
                    $loadingSpan.after(that._renders.errors(AJS.params.attachmentsUnknownError));
                }
                $fileInputContainer.addClass("error");
                this.cleanupAfterSubmit();
            },
            success: function (response) {
                if (response.id) {
                    $loadingSpan.after(that._renders.temporaryFileCheckbox(response));
                } else if (response.errorMsg) {
                    $loadingSpan.after(that._renders.errors(response.errorMsg));
                    $fileInputContainer.addClass("error");
                }
                this.cleanupAfterSubmit();
            }
        });
        $attachForm.submit();
    },

    _renders: {

        /**
         * Creates the jQuery object representing the hidden form to be used for posting the temporary
         * attachment to the server
         *
         * @method _renders.attachForm
         * @private
         *
         * @param {String} postUrl - the url to post the form to
         * @return {jQuery} jQuery wrapped HTML element
         *
         */
        attachForm: function (postUrl) {
            return AJS.$("<form method='post' enctype='multipart/form-data' action='" + postUrl + "'/>").hide();
        },

        /**
         * Clones the provided form input and resets its value
         *
         * @method _renders.fileInput
         * @private
         *
         * @param {jQuery} $originalInput - the jQuery input control to clone
         * @return {jQuery} jQuery wrapped HTML element
         *
         */
        fileInput: function($originalInput) {
            return $originalInput.clone().val("");
        },

        /**
         * Creates the jQuery object representing a loading span for the filename provided.
         *
         * @method _renders.loadingSpan
         * @private
         *
         * @param {String} fileName - the fileName for which to show a loading span
         * @return {jQuery} jQuery wrapped HTML element
         *
         */
        loadingSpan: function(fileName) {
            return AJS.$("<div class='loading file'>" + fileName + "</div>");
        },

        /**
         * Creates the jQuery object representing an input checkbox for an uploaded file. Users can check & uncheck
         * this box if the don't want to upload the file after all.
         *
         * @method _renders.temporaryFileCheckbox
         * @private
         *
         * @param {Object} response - the response object from the server
         * @return {jQuery} jQuery wrapped HTML element
         *
         */
        temporaryFileCheckbox: function(response) {
            return AJS.$("<input type='checkbox' class='checkbox' name='filetoconvert' value='" + response.id + "' title='"+response.title+"' checked><label>" + response.name + "</label>");
        },

        /**
         * Creates the jQuery object representing any server errors when uploading a file
         *
         * @method _renders.errors
         * @private
         *
         * @param {String} errorMsg - the error message to display to the user.
         * @return {jQuery} jQuery wrapped HTML element
         *
         */
        errors: function(errorMsg) {
            return AJS.$("<div>" + errorMsg + "</div>");
        }
    }
});

/**
 *
 * jQuery plugin to turn a file input into an inline attachment control.
 * That is when a file is selected, it will be uploaded to the server immediately and create a
 * temporary attachment on the server.  The client will then display a checkbox to allow the user
 * to select this temporary attachment.
 *
 * Note: Delegates to AJS.InlineAttach
 *
 * <h4>Use </h4>
 *
 * // no options
 * AJS.$("#my-container").inlineAttach();
 *
 * // options
 * AJS.$("#my-container").inlineAttach({
 *      issueId: 10021
 * });
 *
 * For full options @see AJS.InlineAttach
 *
 * @module jQuery
 * @param options
 */
jQuery.fn.inlineAttach = function (options) {
    var res = [];

    this.each(function () {
        options = options || {};
        options.element = this;
        var context = AJS.$(this).closest("form");
        options.issueId = AJS.$("input[name=id]", context).val();
        options.projectId = AJS.$("input[name=pid]", context).val();

        res.push(new AJS.InlineAttach(options));
    });

    return res;
};
