/**
 * A manager for different multiselect modes. For example ctrl + [selection] and shift + [selection]
 *
 * @class SelectionManager
 * @namespace AJS
 * @module Controls
 */
AJS.SelectionManager = AJS.Control.extend({

    /**
     * @constructor
     * @param {HTMLElement, String, JQuery} eventDelegator
     */
    init: function (eventDelegator) {

        var instance = this;

        this.schemes = [];

        AJS.$.each(this.schemesClasses, function (i, schemeClass) {

            var scheme = new schemeClass(eventDelegator);

            scheme.setEnabledCallback(function () {
                instance.setActiveScheme.apply(instance, arguments);
            });
            scheme.setDisabledCallback(function () {
                instance.setNoScheme.apply(instance, arguments);
            });

            instance.schemes.push(scheme);
        });
    },

    /**
     * Gets the active multiselect scheme, if there is one.
     *
     * @method getActiveScheme
     * @return {AJS.SelectionScheme}
     */
    getActiveScheme: function () {
        return this.activeScheme;
    },

    /**
     * Sets active selection scheme
     *
     * @method setActiveScheme
     * @param {AJS.SelectionScheme} scheme
     */
    setActiveScheme: function (scheme) {
        this.trigger(this.getCustomEventName("setActiveScheme"));
        this.activeScheme = scheme;
    },

    /**
     * Sets no scheme
     *
     * @method setNoScheme
     */
    setNoScheme: function () {
        this.trigger(this.getCustomEventName("setNoScheme"));
        this.activeScheme = null;
    },

    /**
     * Listens for schemes to be enabled. Attaches events
     *
     * @method enable
     */
    enable: function () {
        AJS.$.each(this.schemes, function () {
            this.listen();
        });
    },

     /**
     * Unbinds events
     *
     * @method disable
     */
    disable: function () {
        AJS.$.each(this.schemes, function () {
            this.stopListening();
        });
    },

    /**
     * Different selection modes
     *
     * @property {Array} schemesClasses
     *
     */
    schemesClasses: [AJS.SelectiveSelectionScheme, AJS.RangeSelectionScheme]

});
