/**
 * An abstract class that listens for particular key events that enables the selection scheme. For instance when ctrl key
 * is down we are in multiselect mode.
 *
 * @class SelectionScheme
 * @module Controls
 * @namespace AJS
 */
AJS.SelectionScheme = AJS.Control.extend({

    /**
     * @contructor
     * @param {HTMLElement, String, JQuery} eventDelegator - element that events will be bound to
     */
    init: function (eventDelegator) {
        this.$eventDelegator = AJS.$(eventDelegator);
    },

    /**
     * Enables selection state 
     *
     * @method enable
     */
    enable: function () {
        this.enabled = true;
        if (this.enabledCallback) {
            this.enabledCallback(this);
        }
    },

    /**
     * Disables selection state 
     *
     * @method disable
     */
    disable: function () {
        this.enabled = false;
        if (this.disabledCallback) {
            this.disabledCallback(this);
        }
    },

    /**
     * Sets the function to be called when state is enabled
     *
     * @method setEnabledCallback
     * @param {Function} callback
     */
    setEnabledCallback: function (callback) {
        this.enabledCallback = callback;
    },

    /**
     * Sets the function to be called when state is disabled
     *
     * @method setDisabledCallback
     * @param {Function} callback
     */
    setDisabledCallback: function (callback) {
        this.disabledCallback = callback;
    },

    /**
     * Listens for keyboard events
     *
     * @method listen
     */
    listen: function () {
        this._assignEvents("delegator", this.$eventDelegator);
    },

    /**
     * Stops listening for keyboard events
     *
     * @method stopListening
     */
    stopListening: function () {
        this._unassignEvents("delegator", this.$eventDelegator);
    },

    /**
     * Determines if the supplied special key is a valid special key for enabling.
     *
     * @method _resolveKey
     * @param {string} specialKey
     */
    _resolveKey: function (specialKey) {
        var key;
        AJS.$.each(this.triggerKeys, function (_, triggerSpecialKey) {
            if (specialKey === triggerSpecialKey) {
                key = triggerSpecialKey;
                return false;
            }
        });
        return key;
    },

    /**
     * Checks wether the keyCode should enable state. If it is valid installs a keyup event to disable state.
     *
     * @method _resolveKey
     * @param {Number} keyCode
     */
    _handleKeyCode: function (keyCode, e) {
        var instance = this;
        if (this._resolveKey(jira.keyboard.specialKeyEntered(e))) {
            this.enable();
            this.$eventDelegator.bind("keyup", function (keyupEvent) {
                if (keyCode === keyupEvent.which && !keyupEvent.ctrlKey) {
                    instance.disable();
                     AJS.$(this).unbind("keyup", arguments.callee); // unbind myself
                }
            });
        }
    },

    _events: {

        delegator: {
            keydown: function (e) {
                this._handleKeyCode(e.which, e);
            },
            keypress: function (e) {
                this._handleKeyCode(e.which, e);
            }
        }
    }

});

/**
 * Selection mode when control or apple key is down
 *
 * @class SelectiveSelectionScheme
 * @module Controls
 * @namespace AJS
 */
AJS.SelectiveSelectionScheme = AJS.SelectionScheme.extend({
    triggerKeys: [jira.keyboard.SpecialKey.CTRL, jira.keyboard.SpecialKey.ALT, jira.keyboard.SpecialKey.META]
});


/**
 * Selection mode when shift key is down
 *
 * @class RangeSelectionScheme
 * @module Controls
 * @namespace AJS
 */
AJS.RangeSelectionScheme = AJS.SelectionScheme.extend({
    triggerKeys: [jira.keyboard.SpecialKey.SHIFT]
});
