
/**
 * A collection of common controls
 * @module Controls
 * @requires AJS, jQuery, jQuery.hotkeys
 */

/**
 * Keyboard commands with syntactic sugar.
 *
 * <pre>
 * <strong>Usage:</strong>
 * AJS.whenIType("gh").or("gd").goTo("/secure/Dashboard.jspa");
 * AJS.whenIType("c").click("#create_link");
 * </pre>
 *
 * @class whenIType
 *
 * @constuctor whenIType
 * @namespace AJS
 * @param keys - Key combinations, modifier keys are "+" deliminated. e.g "a+b"
 */

jQuery(document).bind("iframeAppended", function (e, iframe) {
    jQuery(iframe).load(function () {
        // Bubble key events from iframes to the main window.
        jQuery(iframe).contents().bind("keyup keydown keypress", function (event) {
            if (!jQuery(event.target).is(":input")) {
                jQuery(document).trigger(event);
            }
        });
    });
});

(function() {
    AJS.whenIType = function(keys) {
        return new ActionController(keys);
    };

    function ActionController(keys) {
        this._executer = null;
        this._keys = jQuery.isArray(keys) ? keys : [keys];
        this._bindKeys(this._keys);
    }

    ActionController.prototype._bindKeys = function(keys) {
        var simple = jQuery.grep(keys, function (key) { return /^[a-z0-9?]$/i.test(key); } ).length === keys.length;
        var that = this;
        jQuery(document).bind(simple ? 'keypress' : 'keydown', keys, function (e) {
            // TODO: Remove dependency on jQuery.hotkeys plugin, then remove jQuery.hotkeys plugin.
            //if (AJS.$.inArray(jira.keyboard.keyEntered(e), keys) >= 0) {
            if (that._executer && !AJS.popup.current && !AJS.dropDown.current && !AJS.FlexiPopup.current) {
                //JRADEV-2323: If there's an inline dialog shown - hide it, since you'll loose focus on the inline
                //dialog anyway as soon as keys are pressed.
                if(AJS.InlineDialog.current) {
                    AJS.InlineDialog.current.hide();
                }
                that._executer(e);
                e.preventDefault();
            }
        });
    };

    ActionController.prototype._addShortcutTitle = function(selector) {
        var elem = jQuery(selector),
            title = elem.attr("title") || "",
            keyArr = this._keys.slice(0);

        if (elem.data("kbShortcutAppended")) {
            this._appendShortcutTitle(elem, keyArr, title);
            return;
        }

        title += " ( " + AJS.params.keyType + " '" + keyArr.shift() + "'";
        jQuery.each(keyArr, function () {
            title += " " + AJS.params.keyThen + " '" + this + "'";
        });
        title += " )";
        elem.attr("title", title);
        elem.data("kbShortcutAppended", true);
    };

    ActionController.prototype._appendShortcutTitle = function(elem, keyArr, title) {
        title = title.replace(/\)$/, " OR ");
        title += "'" + keyArr.shift() + "'";
        jQuery.each(keyArr, function () {
            title += " " + AJS.params.keyThen + " '" + this + "'";
        });
        title += " )";
        elem.attr("title", title);
    };

    /**
     * Scrolls to and adds <em>focused</em> class to the next item in the jQuery collection
     *
     * @method moveToNextItem
     * @param selector
     */
    ActionController.prototype.moveToNextItem = function(selector) {
        this._executer = function () {
            var index,
                items = jQuery(selector),
                focusedElem = jQuery(selector + ".focused");

            if (!this._executer.blurHandler) {
                jQuery(document).one("keypress", function (e) {
                    if (e.keyCode === 27 && focusedElem) {
                        focusedElem.removeClass("focused");
                    }
                });
            }

            if (focusedElem.length === 0) {
                focusedElem = jQuery(selector).eq(0);
            } else {
                focusedElem.removeClass("focused");
                index = jQuery.inArray(focusedElem.get(0), items);
                if (index < items.length-1) {
                    index = index +1;
                    focusedElem = items.eq(index);
                } else {
                    focusedElem.removeClass("focused");
                    focusedElem = jQuery(selector).eq(0);
                }
            }
            if (focusedElem && focusedElem.length > 0) {
                focusedElem.addClass("focused");
                focusedElem.scrollIntoView();
                focusedElem.find("a:first").focus();
            }
        };
    };

    /**
     * Scrolls to and adds <em>focused</em> class to the previous item in the jQuery collection
     *
     * @method moveToPrevItem
     * @param selector
     */
    ActionController.prototype.moveToPrevItem = function(selector) {
        this._executer = function () {
            var index,
                items = jQuery(selector),
                focusedElem = jQuery(selector + ".focused");

            if (!this._executer.blurHandler) {
                jQuery(document).one("keypress", function (e) {
                    if (e.keyCode === 27 && focusedElem) {
                        focusedElem.removeClass("focused");
                    }
                });
            }

            if (focusedElem.length === 0) {
                focusedElem = jQuery(selector + ":last");
            } else {

                focusedElem.removeClass("focused");
                index = jQuery.inArray(focusedElem.get(0), items);
                if (index > 0) {
                    index = index -1;
                    focusedElem = items.eq(index);
                } else {
                    focusedElem.removeClass("focused");
                    focusedElem = jQuery(selector + ":last");
                }
            }
            if (focusedElem && focusedElem.length > 0) {
                focusedElem.addClass("focused");
                focusedElem.scrollIntoView();
                focusedElem.find("a:first").focus();
            }
        };
    };

    /**
     * Clicks the element specified by the <em>selector</em> argument.
     *
     * @method click
     * @param selector - jQuery selector for element
     */
    ActionController.prototype.click = function(selector) {
        this._addShortcutTitle(selector);

        this._executer = function () {
            jQuery(selector).click();
        };
    };

    /**
     * Navigates to specified <em>location</em>
     *
     * @method goTo
     * @param {String} location - http location
     */
    ActionController.prototype.goTo = function(location) {
        this._executer = function () {
            window.location.href = contextPath + location;
        };
    };

    /**
     * navigates browser window to link href
     *
     * @method followLink
     * @param selector - jQuery selector for element
     */
    ActionController.prototype.followLink = function(selector) {
        this._addShortcutTitle(selector);

        this._executer = function () {
            var elem = jQuery(selector);
            if (elem.length > 0 &&
                    (elem.attr("nodeName").toLowerCase() === "a" || elem.attr("nodeName").toLowerCase() === "link")) {
                elem.click();
                window.location.href = elem.attr("href");
            }
        };
    };

    /**
     * Scrolls to element if out of view, then clicks it.
     *
     * @method moveToAndClick
     * @param selector - jQuery selector for element
     */
    ActionController.prototype.moveToAndClick = function(selector) {
        this._addShortcutTitle(selector);

        this._executer = function () {
            var elem = jQuery(selector);
            if (elem.length > 0) {
                elem.click();
                elem.scrollIntoView();
            }
        };
    };

    /**
     * Scrolls to element if out of view, then focuses it
     *
     * @method moveToAndFocus
     * @param selector - jQuery selector for element
     */
    ActionController.prototype.moveToAndFocus = function(selector) {
        this._addShortcutTitle(selector);

        this._executer = function (e) {
            var $elem = jQuery(selector);
            if ($elem.length > 0) {
                $elem.focus();
                $elem.scrollIntoView();
                if ($elem.is(':input')) {
                    e.preventDefault();
                }
            }
        };
    };

    /**
     * Executes the javascript provided by the shortcut plugin point on page load.
     *
     * @method evaluate
     * @param {Function} command - the function provided by the shortcut key plugin point
     */
    ActionController.prototype.evaluate = function(command) {
        if (typeof command !== "function") {
            command = new Function(command);
        }
        command.call(this);
    };

    /**
     * Executes function
     *
     * @method execute
     * @param {function} func
     */
    ActionController.prototype.execute = function(func) {
        var self = this;
        this._executer = function() {
            if (typeof func !== "function") {
                func = new Function(func);
            }
            func.call(self);
        };
    };

    /**
     * Binds additional keyboard controls
     *
     * @method or
     * @param {String} keys - keys to bind
     * @return {Object}
     */
    ActionController.prototype.or = function(keys) {
        this._bindKeys(keys);
        return this;
    };

})();
