(function($) {

    /**
     * Represents an the Issue Navigator page.  This class should be used to retrieve information from the
     * issue navigator such as the currently selected row, currently selected issue key and so on.
     *
     * @class jira.app.issuenavigator
     * @module Domain
     * @requires AJS
     */
    $.namespace("jira.app.issuenavigator");
    var jiraAppIssuenavigator = jira.app.issuenavigator;


    /**
     * Checks if we are currently viewing the issue navigator.
     *
     * @method isNavigator
     * @return {Boolean} true if the current page is the issue navigator, false otherwise
     */
    jiraAppIssuenavigator.isNavigator = function() {
        return $("#isNavigator").length === 1;
    };

    /**
     * Checks if any row is currently selected on the issue navigator. This can be the case for
     * an empty searchr, or if keyboard shortcuts are disabled.
     *
     * @method isRowSelected
     * @return {Boolean} true if a selected issue row exists, false otherwise
     */
    jiraAppIssuenavigator.isRowSelected = function() {
        return this.get$focusedRow().length !== 0;
    };

    /**
     * Returns a jQuery wrapped object representing the currently selected issue row.
     *
     * @method getFocusedRow
     * @return {jQuery} the jQuery wrapped issue row representing the currently selected row
     */
    jiraAppIssuenavigator.get$focusedRow = function() {
        return $("#issuetable tr.issuerow.focused");
    };

    /**
     * Gets the index of the focused issue.
     *
     * @method getFocsuedIssueIndex
     * @return {Number} - The index of the focused issue in the current search result set.
     */
    jiraAppIssuenavigator.getFocsuedIssueIndex = function() {
        var rowIndex = $("#issuetable").find("tr.issuerow").index(jiraAppIssuenavigator.get$focusedRow());
        var searchOffset = parseInt($('#results-count-start').text(), 10) - 1;
        return rowIndex + searchOffset;
    };

    /**
     * Returns the issue key for the currently selected row.
     *
     * @method getSelectedIssueKey
     * @return {String} The issue key for the currently focused row or undefined if none exists.
     */
    jiraAppIssuenavigator.getSelectedIssueKey = function() {
        var $focusedRow = this.get$focusedRow();
        if($focusedRow.length !== 0) {
            return $focusedRow.attr("data-issuekey");
        }
        return undefined;
    };

    /**
     * Returns the issue id for the currently selected row.
     *
     * @method getSelectedIssueId
     * @return {String} The issue id for the currently focused row or undefined if none exists.
     */
    jiraAppIssuenavigator.getSelectedIssueId = function() {
        return this.get$focusedRow().attr("rel");
    };

    /**
     * Returns the issue id for the next row after the currently selected row.
     *
     * Note: It is a known issue that no id will be returned when the last issue on the page is
     * focused. In future, the return value in this situation may change.
     *
     * @method getNextIssueId
     * @return {String} The issue id for the next issue after the currently focused row or undefined if none exists.
     */
    jiraAppIssuenavigator.getNextIssueId = function() {
        return this.get$focusedRow().next("tr.issuerow").attr("rel");
    };
})(AJS.$);

